"""Main module.
Helper functions for the dashboard"""

import altair as alt
from vega_datasets import data
import pandas as pd
import os


def read_jhu_covid_df(ts_folder, name):
    filename = os.path.join(ts_folder, f"time_series_19-covid-{name}.csv")
    df = pd.read_csv(filename)
    df = df.set_index(['Province/State', 'Country/Region', 'Lat', 'Long'])
    df.columns = pd.to_datetime(df.columns)
    return df


def read_jhu_frames_map(ts_folder):

    jhu_frames_map = {
        "confirmed": read_jhu_covid_df(ts_folder, "Confirmed"),
        "deaths": read_jhu_covid_df(ts_folder, "Deaths"),
        "recovered": read_jhu_covid_df(ts_folder, "Recovered")
    }
    return jhu_frames_map


def read_rates_covid_df(rates_folder, name):
    filename = os.path.join(rates_folder, f"ts_rates_19-covid-{name}.csv")
    df = pd.read_csv(filename).drop("Unnamed: 0", axis=1)
    df = df.set_index(['Country/Region'])
    df.columns = pd.to_datetime(df.columns)
    return df


def read_rates_frames_map(rates_folder):
    rates_frames_map = {
        "confirmed": read_rates_covid_df(rates_folder, "confirmed"),
        "deaths": read_rates_covid_df(rates_folder, "deaths"),
        "recovered": read_rates_covid_df(rates_folder, "recovered")
    }
    return rates_frames_map


def read_geodata(geodata_path):
    geodata_df = pd.read_csv(geodata_path)
    geodata_df = geodata_df.drop('Unnamed: 0', axis=1)
    geodata_df = geodata_df.rename({'name_jhu':'Country/Region'}, axis=1)
    geodata_df = geodata_df.set_index('Country/Region')
    return geodata_df


def latest_jhu_country_ser(jhu_frames_map, name):
    return jhu_frames_map[name].iloc[:,-1].groupby(level='Country/Region').sum()


def countries_with_number_of_cases(jhu_frames_map, name, count):
    case_count_ser = latest_jhu_country_ser(jhu_frames_map, 'confirmed')
    countries_over_thresh = case_count_ser[case_count_ser > count - 1].index
    return countries_over_thresh


def latest_rates_ser(rates_frames_map, name):
    return rates_frames_map[name].iloc[:,-1]


def compute_map_df(rates_frames_map, jhu_frames_map, geodata_df, countries_over_thresh):
    map_df = pd.concat([
        latest_rates_ser(rates_frames_map, 'confirmed'),
        latest_rates_ser(rates_frames_map, 'deaths'),
        latest_rates_ser(rates_frames_map, 'recovered')], axis=1)
    nominal_df = pd.concat([
        latest_jhu_country_ser(jhu_frames_map, 'confirmed'),
        latest_jhu_country_ser(jhu_frames_map, 'deaths'),
        latest_jhu_country_ser(jhu_frames_map, 'recovered')], axis=1)
    map_df = pd.concat([map_df, nominal_df, geodata_df[['Longitude', 'Latitude']]], axis=1)
    # Restrict to countries with 100 or more cases
    map_df = map_df.loc[countries_over_thresh].dropna()
    map_df = map_df.reset_index()
    map_df.columns = ['Country/Region', 
                      'Confirmed/100k', 'Deaths/100k', 'Recovered/100k', 
                      'Confirmed', 'Deaths', 'Recovered',
                      'Long', 'Lat']
    return map_df


def map_of_variable(map_df, variable, title):
    # Data generators for the background
    sphere = alt.sphere()
    graticule = alt.graticule()

    # Source of land data
    source = alt.topo_feature(data.world_110m.url, 'countries')

    # Layering and configuring the components
    p = alt.layer(
        alt.Chart(sphere).mark_geoshape(fill='#cae6ef'),
        alt.Chart(graticule).mark_geoshape(stroke='white', strokeWidth=0.5),
        alt.Chart(source).mark_geoshape(fill='#dddddd', stroke='#aaaaaa'),
        alt.Chart(map_df).mark_circle(opacity=0.6).encode(
            longitude='Long:Q',
            latitude='Lat:Q',
            size=alt.Size(f'{variable}:Q', title="Cases"),
            color=alt.value('steelblue'),
            tooltip=["Country/Region:N", 
                     "Confirmed:Q", "Deaths:Q", "Recovered:Q",
                     "Confirmed/100k:Q", "Deaths/100k:Q", "Recovered/100k:Q"]
        )
    ).project(
        'naturalEarth1'
    ).properties(width=600, height=400, title=f"{title} cases per 100k inhabitants"
    ).configure_view(stroke=None)
    return p


def growth_df(rates_frames_map, geodata_df, name, countries_over_thresh, cutoff):
    latest_confirmed_ser = rates_frames_map['confirmed'].iloc[:,-1]
    countries_over_1 = latest_confirmed_ser[latest_confirmed_ser >= cutoff].reset_index()['Country/Region']

    confirmed_rate_df = rates_frames_map['confirmed']
    confirmed_rate_df = confirmed_rate_df.loc[
        confirmed_rate_df.index.isin(countries_over_1) &
        confirmed_rate_df.index.isin(countries_over_thresh)]
    confirmed_rate_df = confirmed_rate_df.join(
        geodata_df[['Longitude', 'Latitude', 'region_un']]).set_index(
        ['Longitude', 'Latitude', 'region_un'], append=True)
    confirmed_rate_df = confirmed_rate_df.stack().reset_index()
    confirmed_rate_df = confirmed_rate_df.rename(
        {'region_un': 'Geo Region', 'level_4': 'Date', 0: 'Confirmed/100k'}, axis=1)
    return confirmed_rate_df
